package cn.estsh.i3plus.pojo.aps.bean;

import cn.estsh.i3plus.pojo.aps.annotation.ExcludeImportExport;
import cn.estsh.i3plus.pojo.aps.annotation.FieldAnnotation;
import cn.estsh.i3plus.pojo.aps.common.BaseAPS;
import cn.estsh.i3plus.pojo.aps.common.BeanRelation;
import cn.estsh.i3plus.pojo.aps.holders.EParentWork;
import com.fasterxml.jackson.annotation.JsonBackReference;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiParam;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import java.util.List;

/**
 * @Description :父工作
 * 父工作用于记录工作分割前的信息。
 * 生产订单应关联父工作，父工作关联多个工作。
 * @Reference :
 * @Author : jason.niu
 * @CreateDate : 2019-09-17
 * @Modify:
 **/
@Data
@EqualsAndHashCode(callSuper = true)
@Entity
@Table(name = "APS_PARENT_WORK")
@Api("父工作")
@ExcludeImportExport
public class ParentWork extends BaseAPS {
    private static final long serialVersionUID = 8792781216193806606L;

    @Column(name="count", columnDefinition = "decimal(18,8)")
    @ApiParam(value ="数量")
    private Double count;

    @Column(name="SERIAL_NUMBER")
    @ApiParam(value ="拆分工作流水号")
    @FieldAnnotation(property = false)
    private Integer serialNumber = 0;

    @Column(name="CONFIRM_SERIAL")
    @ApiParam(value ="下发流水号")
    @FieldAnnotation(property = false)
    private Integer confirmSerial = 0;

    @Column(name="POST_SERIAL_NUMBER")
    @ApiParam(value ="子工作流水号")
    @FieldAnnotation(property = false)
    private Integer postSerialNumber = 0;

    @Column(name="NEW_SERIAL")
    @ApiParam(value ="拆分工作流水号")
    @FieldAnnotation(property = false)
    private Boolean newSerial = false;

    @Column(name="PRODUCT_ORDER_ID")
    @ApiParam(value ="订单")
    @FieldAnnotation(property = false)
    private Long productOrderId;

    @Column(name="OPERATION_ID")
    @ApiParam(value ="工序")
    @FieldAnnotation(property = false)
    private Long operationId;

//    public int getCodeNumber() {
//        for (int bitIndex = 0; bitIndex < 10; ++bitIndex) {
//            int value = 1 << bitIndex;
//            if ((this.serialNumber & value) == 0) {
//                this.serialNumber |= value;
//                newSerial = true;
//                return bitIndex;
//            }
//        }
//
//        return 0;
//    }
//
//    public void recoverNumber(int value) {
//        if (newSerial == null || !newSerial) {
//            return;
//        }
//
//        value = ~(1 << value);
//        this.serialNumber &= value;
//    }

    public BaseOrder getProductOrder() {
        return BeanRelation.get(this, EParentWork.ProductOrder);
    }

    public void setProductOrder(ProductOrder order) {
        this.productOrderId = order != null ? order.getId() : 0L;
        BeanRelation.set(this, EParentWork.ProductOrder, order);
    }

    public Operation getOperation() {
        return BeanRelation.get(this, EParentWork.Operation);
    }

    public void setOperation(Operation oper) {
        this.operationId = oper != null ? oper.getId() : 0L;
        BeanRelation.set(this, EParentWork.Operation, oper);
    }

    @JsonBackReference
    public List<Work> getWorks() {
        return BeanRelation.list(this, EParentWork.Works);
    }

    public int createNew() {
        return getPrevNumber() * 10;
    }

    public int createContinue(int number) {
        number /= 10;
        return number * 10 + getPostNumber();
    }

    private int getPrevNumber() {
        for (int bitIndex = 1; bitIndex < 10; ++bitIndex) {
            int value = 1 << bitIndex;
            if ((this.serialNumber & value) == 0) {
                this.serialNumber |= value;
                this.newSerial = true;
                return bitIndex ;
            }
        }

        return 0;
    }

    private int getPostNumber() {
        for (int bitIndex = 1; bitIndex < 10; ++bitIndex) {
            int value = 1 << bitIndex;
            if ((this.postSerialNumber & value) == 0) {
                this.postSerialNumber |= value;
                return bitIndex;
            }
        }

        return 0;
    }

    public void cycleNumber(int number) {
        if (newSerial == null || !newSerial) {
            return;
        }

        cyclePrevNumber(number);
        cyclePostNumber(number);
    }

    private void cyclePrevNumber(int number) {
        if (number % 10 > 0) {
            return;
        }

        number /= 10;
        number = ~(1 << number);
        this.serialNumber &= number;
    }

    private void cyclePostNumber(int number) {
        number %= 10;
        if (number == 0) {
            return;
        }
        if (this.postSerialNumber == null) {
            this.postSerialNumber = 0;
        }

        number = ~(1 << number);
        this.postSerialNumber &= number;
    }
}
